/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * BGNEstimator.hpp
 *
 *  Created on: 2010/12/09
 *      Author: HARK Development Team
 */

#ifndef BGNESTIMATOR_HPP_
#define BGNESTIMATOR_HPP_

#include <vector>
#include <boost/shared_ptr.hpp>
#include <LRSignal.hpp>
#include <Logger.hpp>

namespace NSHarkBinaural
{

/**
 * バックグラウンドノイズ見積り器.
 * 音声の初めの方を使ってバックグラウンドノイズを見積もる。
 */
class BGNEstimator
{
public:
    /**
     * バックグラウンドノイズ見積り器を生成する.
     * @param estimateCount 見積りに使うスペクトルの数。estimateCount回だけEstimate()を呼び出すと見積りが完了する
     */
    BGNEstimator(unsigned int estimateCount = 10);

    /**
     * バックグラウンドノイズを見積もる.
     * 信号がシフトするごとに呼び出す必要がある。
     * ある程度の回数だけ呼び出すと見積りが完了し、Estimated()がtrueとなる。
     * @param lrspectra 左右スペクトル
     */
    void Estimate(LRSpectra::ptr_t lrspectra);

    /**
     * バックグラウンドノイズの見積りが完了したかどうか.
     * @return 見積りが完了しているならtrue
     */
    bool Estimated()
    {
        return this->estimated;
    }

    /**
     * バックグラウンドノイズを得る.
     * Estimated()がtrueにならないうちに呼び出すと、nullを返す。
     * @return 見積もられたバックグラウンドノイズ（スペクトル）
     */
    LRSpectra::ptr_t BGNoiseSpectra();

    /**
     * バックグラウンドノイズを得る.
     * Estimated()がtrueにならないうちに呼び出すと、nullを返す。
     * @return 見積もられたバックグラウンドノイズ（パワースペクトル）
     */
    LRUnspectra::ptr_t BGNoise();

private:
    /**
     * 見積り回数カウンタ.
     */
    unsigned int estimateCounter;

    /**
     * 見積り回数
     */
    const unsigned int estimateCount;

    /**
     * 見積りが完了したかどうか.
     * 完了したら、それ以降はノイズを取り出して利用できる。
     */
    bool estimated;

    /**
     * バックグラウンドノイズ.
     * スペクトルにより計算されたもの。
     */
    LRSpectra::ptr_t bgnoiseSpectra;

    /**
     * バックグラウンドノイズ.
     * パワースペクトルにより計算されたもの。
     */
    LRUnspectra::ptr_t bgnoisePowerSpectra;
};

}

#endif /* BGNESTIMATOR_HPP_ */
